# Argumentos con los que se llama el script, se espera que el 1 argumento sean las secuencias mir con su nombre y el 2 las secuencias mrna con el ensembleid


args <- commandArgs(trailingOnly = TRUE)
mirsec <- read.csv(args[1], header = TRUE) 
mrnasec <- read.csv(args[2], header = TRUE)
mrnasec["id"] <- c(1:nrow(mrnasec)) 

# carga/instalacion de los paquetes necesarios
if (!require(MASS)) install.packages("MASS")
if (!require(seqinr)) install.packages("seqinr")
if (!require(dplyr)) install.packages("dplyr")
if (!require(multiMiR)) install.packages("multiMir")
if (!require(caret)) install.packages("caret")

# Carpetas que contienen los clasificadores(modificar)

dir_miranda = "/home/antonio/TFM/Clasificadores/miRanda-3.3a"
dir_tarpmir = "/home/antonio/TFM/Clasificadores/TarPmiR"
dir_intarna = "/home/antonio/IntaRNA-master"
dir_clasificador = "/home/antonio"


resultados <- data.frame(mirsec[,1],mrnasec[,1])


# 1º Clasificador TARPMIR

setwd(dir_tarpmir)

# Se crean 2 archivos fasta en la carpeta de TARPMIR para que pueda funcionar

write.fasta(as.list(mirsec[,2]), names = mirsec[,1], file.out = "mirsec.txt")
write.fasta(as.list(mrnasec[,2]), names = mrnasec$id, file.out = "mrnasec.txt")

system("python TarPmiR.py -a mirsec.txt -b mrnasec.txt -m models/Human_sklearn_0.18.pkl -p 0.0")

# Tratamiento de los resultados

resultados1tpm <- read.table("mirsec.txt_mrnasec.txt.bp")

resultados["rtpm"] <- NA

for (i in 1:nrow(resultados1tpm)) {
  for (j in 1:nrow(resultados)){
    if (resultados1tpm$V1[i] == resultados[,1][j] & resultados1tpm$V2[i] == resultados[,2][j]){
      resultados$rtpm[j] <- resultados1tpm$V4[i]
    }
  }
}


# 2 Clasificador miRanda

setwd(dir_miranda)

# se vuelven a escribir los mismos archivos en la carpeta de miranda

write.fasta(as.list(mirsec[,2]), names = mirsec[,1], file.out = "mirsec.txt")
write.fasta(as.list(mrnasec[,2]), names = mrnasec[,1], file.out = "mrnasec.txt")

system("miranda mirsec.txt mrnasec.txt -out result.txt")

resultadosMRD <- read.table("result.txt", fill = TRUE)
resultadosMRD <- resultadosMRD[-(1:27),]


#Búsqueda sólo de positivos
mirs_positivemrd <- as.vector(resultadosMRD$V1[which(grepl("hsa", resultadosMRD$V1) == TRUE)])
id_positivemrd <- as.vector(resultadosMRD$V2[which(grepl("hsa", resultadosMRD$V1) == TRUE)])

# Busqueda de negativos
mirs_negativemrd <- c()
id_negativemrd <- c()


for (i in 1:nrow(resultadosMRD)){
  if (resultadosMRD$V1[i] == "Performing" & resultadosMRD$V1[i+3] == "No"){
    mirs_negativemrd <- c(mirs_negativemrd, resultadosMRD$V3[i])
    id_negativemrd <- c(id_negativemrd, resultadosMRD$V5[i])
  }
}
# Eliminar los >
mirs_positivemrd <-  gsub( ">", "", as.character(mirs_positivemrd))
mirs_negativemrd <- gsub( ">", "", as.character(mirs_negativemrd))



# Unificar los resultados
a <- data.frame(mirs_positivemrd,id_positivemrd)
colnames(a) <- c("mir","id")
a["result"] <- 1
b <- data.frame(mirs_negativemrd, id_negativemrd)
colnames(b) <- c("mir","id")
b["result"] <- 0
allresultsRMD1 <- rbind(a,b)

resultados["rmrd"] <- NA

for (i in 1:nrow(resultados)){
  for (j in 1:nrow(allresultsRMD1)){
    if (resultados[,1][i] == allresultsRMD1$mir[j] & resultados[,2][i] == allresultsRMD1$id[j]){
      resultados$rmrd[i] <- allresultsRMD1$result[j]
    }
  }
}


# Clasificador Inta RNA

setwd(dir_intarna)

write.fasta(as.list(mirsec[,2]), names = mirsec[,1], file.out = "mirsec.txt")
write.fasta(as.list(mrnasec[,2]), names = mrnasec[,1], file.out = "mrnasec.txt")

system("IntaRNA -t mrnasec.txt -q mirsec.txt > results1.txt")

resultsIRNA1 <- read.table("results1.txt", fill = TRUE)


# Busqueda de negativos
nomrna <- c()
nomirna <- c()
for (i in 1:nrow(resultsIRNA1)){
  if (resultsIRNA1$V1[i] == "no"){
    nomrna <- c(nomrna,resultsIRNA1$V2[i+1])
    nomirna <- c(nomirna,resultsIRNA1$V1[i+2])
  }
}

# Busqueda de positivos
pmrna <- c()
pmirna <- c ()

for (i in 10:nrow(resultsIRNA1)){
  if (grepl("ENSG", resultsIRNA1$V1[i]) == TRUE ){
    pmrna <- c(pmrna, resultsIRNA1$V1[i])
    pmirna <- c(pmirna, resultsIRNA1$V1[i+10])
  }
}


# Union de los resultados
prirna <- data.frame(pmrna,pmirna)
colnames(prirna) <- c("mir", "id")
prirna["result"] <- 1
nrirna <- data.frame(nomrna,nomirna)
colnames(nrirna) <- c("mir", "id")
nrirna["result"] <- 0
rIRNA1 <- rbind(prirna,nrirna)
rIRNA1 <- distinct(rIRNA1)
colnames(rIRNA1) <- c("id","mir" ,"result")

resultados["rirna"] <- NA
for (i in 1:nrow(resultados)){
  for (j in 1:nrow(rIRNA1)){
    if (resultados[,2][i] == rIRNA1$id[j] & resultados[,1][i] == rIRNA1$mir[j]){
      resultados$rirna[i] <- rIRNA1$result[j]
    }
  }
} 


# Busqueda en multimir con los ensembl id

ensembl1 <- as.vector(resultados[,2])

dfmultimir <- get_multimir(org = "hsa",
                          target = ensembl1,
                          table = "predicted")



# Ahora se dividen los datos en diferentes dataframes para considerarlos como predictores independientes
dfmultimir <- dfmultimir@data

dfmultimir[dfmultimir==""] <- NA
dfmultimir <- na.omit(dfmultimir)

dmtmultimir <- dfmultimir[which(dfmultimir$database == "diana_microt"),]
elmomultimir <- dfmultimir[which(dfmultimir$database == "elmmo"),]
mccmultimir <- dfmultimir[which(dfmultimir$database == "microcosm"),]
mrdmultimir <- dfmultimir[which(dfmultimir$database == "miranda"),]
pictarmultimir <- dfmultimir[which(dfmultimir$database == "pictar"),]
pitamultimir <- dfmultimir[which(dfmultimir$database == "pita"),]
mdbmultimir <- dfmultimir[which(dfmultimir$database == "mirdb"),]
tsmultimir <- dfmultimir[which(dfmultimir$database == "targetscan"),]

# Se añade la columna de resultados de los predictores de multimir
resultados["rmmdmt"] <- NA
resultados["rmmelm"] <- NA
resultados["rmmcc"] <- NA
resultados["rmmpic"] <- NA
resultados["rmmpita"] <- NA
resultados["rmmts"] <- NA

print("separado de datos")

for (i in 1:nrow(resultados)){
  for (j in 1:nrow(elmomultimir)){
    if (elmomultimir$mature_mirna_id[j] == resultados[,1][i] & elmomultimir$target_ensembl[j] == resultados[,2][i]){
      resultados$rmmelm[i] <- elmomultimir$score[j]
    }
  }
}
for (i in 1:nrow(resultados)){
  for (j in 1:nrow(dmtmultimir)){
    if (dmtmultimir$mature_mirna_id[j] == resultados[,1][i] & dmtmultimir$target_ensembl[j] == resultados[,2][i]){
      resultados$mmdmt[i] <- dmtmultimir$score[j]
    }
  }
}
for (i in 1:nrow(resultados)){
  for (j in 1:nrow(mccmultimir)){
    if (mccmultimir$mature_mirna_id[j] == resultados[,1][i] & mccmultimir$target_ensembl[j] == resultados[,2][i]){
      resultados$rmmcc[i] <- mccmultimir$score[j]
    }
  }
}
for (i in 1:nrow(resultados)){
  for (j in 1:nrow(pictarmultimir)){
    if (pictarmultimir$mature_mirna_id[j] == resultados[,1][i] & pictarmultimir$target_ensembl[j] == resultados[,2][i]){
      resultados$rmmpic[i] <- pictarmultimir$score[j]
    }
  }
}
for (i in 1:nrow(resultados)){
  for (j in 1:nrow(pitamultimir)){
    if (pitamultimir$mature_mirna_id[j] == resultados[,1][i] & pitamultimir$target_ensembl[j] == resultados[,2][i]){
      resultados$rmmpita[i] <- pitamultimir$score[j]
    }
  }
}
for (i in 1:nrow(resultados)){
  for (j in 1:nrow(tsmultimir)){
    if (tsmultimir$mature_mirna_id[j] == resultados[,1][i] & tsmultimir$target_ensembl[j] == resultados[,2][i]){
      resultados$rmmts[i] <- tsmultimir$score[j]
    }
  }
}


resultados[is.na(resultados)] <-0 # Todos los NA se pasan a 0 para poder alimentar el modelo

feedmodelo <- resultados[,-c(1,2)] # se crea un dataset exclusivamente con resultados obtenidos.

for (i in 1:ncol(feedmodelo)){
  feedmodelo[,i] <- as.numeric(feedmodelo[,i])
}

setwd(dir_clasificador)

modelo <- readRDS("modelo_knn1.rds")



predicciones <- predict(modelo, feedmodelo)

predicciones <- as.data.frame(predicciones)

predicciones["Ensembl_id"] <- mrnasec[,1]
predicciones["mir_id"] <- mirsec[,1]


# los resultados se devuelven en un archivo .txt y se muestran en pantalla
print(predicciones)
write.csv(predicciones, "predicciones.txt")







